#-*- perl -*-
#-*- coding: us-ascii -*-

package CSS::Yamaantaka::MirrorH;

use strict;
#use warnings;
use base qw(
    CSS::Yamaantaka::_ReverseLR
    CSS::Yamaantaka::_Mirror
    CSS::Yamaantaka::_ReverseGD
    CSS::Yamaantaka::_NoSwapHV
);

1;
